// ==============================================================================
// Copyright 2020 The LatticeX Foundation
// This file is part of the Rosetta library.
//
// The Rosetta library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// The Rosetta library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with the Rosetta library. If not, see <http://www.gnu.org/licenses/>.
// ==============================================================================
#include "helix_impl_util.h"

#include <iostream>
#include <string>
#include <vector>

using namespace std;
namespace rosetta {
int HelixOpsImpl::Reveal(const vector<string>& a, vector<string>& c, const attr_type* attr_info) {
  string nodes = get_attr_value(attr_info, "receive_parties", "");
  vector<Share> shareA;
  helix_convert_string_to_share(a, shareA);
  AUDIT("id:{}, P{} Reveal input X(Share){}", _op_msg_id.get_hex(),  hi->party_id(), Vector<Share>(shareA));

  vector<double> fc;
  hi->Reveal(shareA, fc, nodes);
  helix_double_to_plain_string(fc, c);
  AUDIT("id:{}, P{} Reveal output(double, plain){}", _op_msg_id.get_hex(), hi->party_id(), Vector<double>(fc));

  return 0;
}

int HelixOpsImpl::Reveal(const vector<string>& a, vector<double>& c, const attr_type* attr_info) {
  string nodes = get_attr_value(attr_info, "receive_parties", "");
  vector<Share> shareA;
  helix_convert_string_to_share(a, shareA);
  AUDIT("id:{}, P{} Reveal input X(Share){}", _op_msg_id.get_hex(),  hi->party_id(), Vector<Share>(shareA));

  hi->Reveal(shareA, c, nodes);
  AUDIT("id:{}, P{} Reveal output(double, plain){}", _op_msg_id.get_hex(), hi->party_id(), Vector<double>(c));

  return 0;
}

/**
 * @desc: This is for Tensorflow's SaveV2 Op.
 */
int HelixOpsImpl::ConditionalReveal(
  vector<string>& in_vec,
  vector<string>& out_cipher_vec,
  vector<double>& out_plain_vec) {
  // not to reveal plaintext by default
  const vector<string>& save_mode = context_->SAVER_MODE;

  // all ciphertext
  if (save_mode.empty()) {
    out_cipher_vec = in_vec;
    out_plain_vec.clear();
    return 0;
  }
  // only plaintext for selected parties
  vector<Share> shareA;
  helix_convert_string_to_share(in_vec, shareA);
  AUDIT("id:{}, P{} Reveal, input(Share){}", _op_msg_id.get_hex(), hi->party_id(),  Vector<Share>(shareA));

  vector<double> c;
  hi->Reveal(shareA, c, save_mode);
  string current_node_id = io->GetCurrentNodeId();
  if (std::find(save_mode.begin(), save_mode.end(), current_node_id) != save_mode.end()) {
    out_plain_vec.swap(c);
  } else {
    out_plain_vec.resize(in_vec.size());
  }
  out_cipher_vec.clear();

  AUDIT("id:{}, P{} Reveal, output(double, plain){}", _op_msg_id.get_hex(), hi->party_id(),  Vector<double>(out_plain_vec));
  //AUDIT("id:{}, Reveal nodes:{}, output{}", _op_msg_id.get_hex(), nodes, Vector<Share>(shareC));

  return 0;
}

} // namespace rosetta
